"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
const index_1 = require("./index");
const autoBind = require("auto-bind");
const graphql_codegen_plugin_helpers_1 = require("graphql-codegen-plugin-helpers");
const changeCase = require("change-case");
class StencilApolloVisitor extends graphql_codegen_visitor_plugin_common_1.ClientSideBaseVisitor {
    constructor(fragments, rawConfig) {
        super(fragments, rawConfig, {
            componentType: graphql_codegen_visitor_plugin_common_1.getConfigValue(rawConfig.componentType, index_1.StencilComponentType.functional)
        });
        autoBind(this);
    }
    getImports() {
        const baseImports = super.getImports();
        const imports = [];
        if (this.config.componentType === index_1.StencilComponentType.class) {
            imports.push(`import { Component } from '@stencil/core';`);
        }
        return [baseImports, ...imports].join('\n');
    }
    _buildOperationFunctionalComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        const operationName = this.convertName(node.name.value);
        const propsTypeName = this.convertName(operationName + 'Props');
        const apolloStencilComponentTag = changeCase.paramCase(`Apollo${operationType}`);
        const onReadySignature = graphql_codegen_plugin_helpers_1.toPascalCase(`On${operationType}ReadyFn`);
        const componentName = this.convertName(`${operationName}Component`);
        const propsVar = `
        export type ${propsTypeName} = {
            variables ?: ${operationVariablesTypes};
            onReady ?: import('stencil-apollo/dist/types/components/${apolloStencilComponentTag}/types').${onReadySignature}<${operationResultType}, ${operationVariablesTypes}>;
        };
      `;
        const component = `
        export const ${componentName} = (props: ${propsTypeName}) => <${apolloStencilComponentTag} ${operationType.toLowerCase()}={ ${documentVariableName} } { ...props } />;
      `;
        return [propsVar, component].filter(a => a).join('\n');
    }
    _buildClassComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        const componentName = this.convertName(node.name.value + 'Component');
        const apolloStencilComponentTag = changeCase.paramCase(`Apollo${operationType}`);
        const onReadySignature = graphql_codegen_plugin_helpers_1.toPascalCase(`On${operationType}ReadyFn`);
        return `
            @Component({
                tag: '${changeCase.paramCase(`Apollo${graphql_codegen_plugin_helpers_1.toPascalCase(node.name.value)}`)}'
            })
            export class ${componentName} {
                @Prop() onReady: import('stencil-apollo/dist/types/components/${apolloStencilComponentTag}/types').${onReadySignature}<${operationResultType}, ${operationVariablesTypes}>;
                render() {
                    return <${apolloStencilComponentTag} ${operationType.toLowerCase()}={ ${documentVariableName} } onReady={ this.onReady } />;
                }
            }
      `;
    }
    buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        switch (this.config.componentType) {
            case index_1.StencilComponentType.class:
                return this._buildClassComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
            case index_1.StencilComponentType.functional:
                return this._buildOperationFunctionalComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
            default:
                return '';
        }
    }
}
exports.StencilApolloVisitor = StencilApolloVisitor;
//# sourceMappingURL=visitor.js.map