import { ClientSideBaseVisitor, getConfigValue } from 'graphql-codegen-visitor-plugin-common';
import { StencilComponentType } from './index';
import * as autoBind from 'auto-bind';
import { toPascalCase } from 'graphql-codegen-plugin-helpers';
import * as changeCase from 'change-case';
export class StencilApolloVisitor extends ClientSideBaseVisitor {
    constructor(fragments, rawConfig) {
        super(fragments, rawConfig, {
            componentType: getConfigValue(rawConfig.componentType, StencilComponentType.functional)
        });
        autoBind(this);
    }
    getImports() {
        const baseImports = super.getImports();
        const imports = [];
        if (this.config.componentType === StencilComponentType.class) {
            imports.push(`import { Component } from '@stencil/core';`);
        }
        return [baseImports, ...imports].join('\n');
    }
    _buildOperationFunctionalComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        const operationName = this.convertName(node.name.value);
        const propsTypeName = this.convertName(operationName + 'Props');
        const apolloStencilComponentTag = changeCase.paramCase(`Apollo${operationType}`);
        const onReadySignature = toPascalCase(`On${operationType}ReadyFn`);
        const componentName = this.convertName(`${operationName}Component`);
        const propsVar = `
        export type ${propsTypeName} = {
            variables ?: ${operationVariablesTypes};
            onReady ?: import('stencil-apollo/dist/types/components/${apolloStencilComponentTag}/types').${onReadySignature}<${operationResultType}, ${operationVariablesTypes}>;
        };
      `;
        const component = `
        export const ${componentName} = (props: ${propsTypeName}) => <${apolloStencilComponentTag} ${operationType.toLowerCase()}={ ${documentVariableName} } { ...props } />;
      `;
        return [propsVar, component].filter(a => a).join('\n');
    }
    _buildClassComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        const componentName = this.convertName(node.name.value + 'Component');
        const apolloStencilComponentTag = changeCase.paramCase(`Apollo${operationType}`);
        const onReadySignature = toPascalCase(`On${operationType}ReadyFn`);
        return `
            @Component({
                tag: '${changeCase.paramCase(`Apollo${toPascalCase(node.name.value)}`)}'
            })
            export class ${componentName} {
                @Prop() onReady: import('stencil-apollo/dist/types/components/${apolloStencilComponentTag}/types').${onReadySignature}<${operationResultType}, ${operationVariablesTypes}>;
                render() {
                    return <${apolloStencilComponentTag} ${operationType.toLowerCase()}={ ${documentVariableName} } onReady={ this.onReady } />;
                }
            }
      `;
    }
    buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        switch (this.config.componentType) {
            case StencilComponentType.class:
                return this._buildClassComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
            case StencilComponentType.functional:
                return this._buildOperationFunctionalComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes);
            default:
                return '';
        }
    }
}
//# sourceMappingURL=visitor.js.map